using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using System.IO;

namespace Nemo
{
    public struct Tuple<TFirst,TSecond> 
	{
        public TFirst first;
        public TSecond second;
        
        public Tuple(TFirst first, TSecond second) 
        { 
			this.first = first;
			this.second = second; 
		}
    }

	class Helpers 
    {
		public static VoidFunction<A> RunInMainThread<A>(VoidFunction<A> b)
		{
			return delegate(A a) { Gtk.Application.Invoke(delegate { b(a); }); };
		}

		public static VoidFunction RunInMainThread(VoidFunction b)
		{
			return delegate { Gtk.Application.Invoke( delegate { b(); }); };
		}
    
		public static Type load<Type>(string filename)
		{
			Type loaded = default(Type);
			XmlSerializer s = new XmlSerializer(typeof(Type));
			TextReader r = new StreamReader(System.IO.Path.Combine(System.IO.Path.Combine(Environment.GetFolderPath(System.Environment.SpecialFolder.Personal), ".nemo/"), filename));
			loaded = (Type)s.Deserialize(r);
			r.Close();
			return loaded;
		}

		public static void save<Type>(string filename, Type to_save)
		{
			XmlSerializer s = new XmlSerializer(typeof(Type));
			System.IO.DirectoryInfo dir = new DirectoryInfo(System.IO.Path.Combine(Environment.GetFolderPath(System.Environment.SpecialFolder.Personal), ".nemo"));
			if (!dir.Exists)
				dir.Create();
			TextWriter w = new StreamWriter(System.IO.Path.Combine(System.IO.Path.Combine(Environment.GetFolderPath(System.Environment.SpecialFolder.Personal), ".nemo/"), filename));
			s.Serialize(w, to_save);
			w.Close();
		}

        public static T[] to_array<T>(List<T> list)
        {
            T[] t = new T[list.Count];
            int index = 0;
            foreach (T element in list) {
                t[index] = element;
                ++index;
            }
            
            return t;
        }

		public static IEnumerable<int> ints()
		{
			int i = 0;
			while (true)
				yield return i++;
		}

		public static IEnumerable<string> readfile(string filename)
		{
		    using (TextReader r = new StreamReader(filename))
            {
				string result;
				while ((result = r.ReadLine()) != null)
				    yield return result;
		    }
		}

        public static IEnumerable<Tuple<T1,T2>> zip<T1, T2>(IEnumerable<T1> l1, IEnumerable<T2> l2)
        {
            IEnumerator<T1> i1 = l1.GetEnumerator();
            IEnumerator<T2> i2 = l2.GetEnumerator();

            while (i1.MoveNext() && i2.MoveNext())
                yield return new Tuple<T1, T2>(i1.Current, i2.Current);
        }

        public static IEnumerable<Tuple<int,T>> enumerate<T>(IEnumerable<T> ie)
        {
        	return zip<int, T>(ints(), ie);
        }

        public static bool sorted_lists_equal<T>(List<T> l1, List<T> l2) where T:IComparable
        {
            if (l1.Count != l2.Count)
                return false;
            foreach (Tuple<T, T> t in zip<T,T>(l1, l2)) {
                if (t.first.CompareTo(t.second) != 0)
                    return false;
            }
            return true;
        }
    }

	public class Color
	{
		public byte r;
		public byte g;
		public byte b;
		public byte a;
		
		public Color(byte r, byte g, byte b)
		{
			this.r = r;
			this.g = g;
			this.b = b;
			this.a = 0;
		}

		// serialization
		public Color()
		{}
	}

    // function templates
    public delegate void VoidFunction();

    public delegate void VoidFunction<A>(A a);
    public delegate void VoidFunction<A, B>(A a, B b);
    public delegate void VoidFunction<A, B, C>(A a, B b, C c);
    public delegate void VoidFunction<A, B, C, D>(A a, B b, C c, D d);

    public delegate R Function<R>();
    public delegate R Function<R, A>(A a);
    public delegate R Function<R, A, B>(A a, B b);
    public delegate R Function<R, A, B, C>(A a, B b, C c);
    public delegate R Function<R, A, B, C, D>(A a, B b, C c, D d);

	public class Singleton<T>
	{
	    private static T _instance;
	    private static readonly object LOCK = new object();

	    public static T Instance
	    {
	        get
	        {
	      		if (_instance == null)
			      lock (LOCK)
	                if (_instance == null)
	                    _instance = (T)System.Activator.CreateInstance(typeof(T), true);
	            return _instance;
	        }
	    }
	}
}
