using System;
using System.Collections.Generic;

using Gtk;

namespace Nemo
{
	public class SearchPopup : Overlay
	{
		Gtk.Window popup;
		Gtk.VBox elements;
		
		VoidFunction<CalendarDriver.View, DateTime> set_view_callback;
		
		public SearchPopup(VoidFunction<CalendarDriver.View, DateTime> set_view)
		{
			popup = null;
			elements = null;
			set_view_callback = set_view;
			name = "search_popup";
		}

		public override void hide()
		{
			popup.Hide();
		}
		
		public override void show()
		{
			popup.Show();
		}
		
		public override void shift(int x, int y)
		{
		}
		
		public override void resize()
		{
		}
		
		public void set_files_and_show(List<Item> items, string search_text, int x, int y)
		{
			create_window(search_text, x, y);

			foreach (Item item in items)
				elements.PackStart(create_element(item, search_text, "", delegate { popup.ShowAll(); }), true, true, 5);

			if (items.Count == 0) {
				Gtk.Alignment tmp = new Gtk.Alignment(0.5f, 1f, 1f, 1f);
				tmp.BorderWidth = 5;
				tmp.Add(new Gtk.Label(Mono.Unix.Catalog.GetString("No elements found")));
				elements.Add(tmp);
			}

			System.Console.WriteLine("elements size {0}", items.Count);

			elements.SizeAllocated += delegate(object sender, Gtk.SizeAllocatedArgs args) {
				popup.Move(x, y - args.Allocation.Height);
			};

			popup.ShowAll();
		}
		
		private void create_window(string search_text, int x, int y)
		{
			System.Console.WriteLine("creating search window");

			if (popup != null)
				popup.Destroy();

			popup = new Gtk.Window(Gtk.WindowType.Popup);
			elements = new VBox();

	        Gtk.Alignment alignment = new Gtk.Alignment(1F, 0.5F, 0F, 0F);
			Gtk.EventBox image_wrapper = new EventBox();
			Gtk.Image close_image = new Gtk.Image(null, "close.png");
			image_wrapper.Add(close_image);

			if (name == "")
				throw new Exception("name for an overlay cannot be empty");

			image_wrapper.ButtonPressEvent += delegate(object sender, Gtk.ButtonPressEventArgs args) { Singleton<OverlayTracker>.Instance.hide_and_die(name); };
			GtkCommon.set_background_color(image_wrapper, "white");
			alignment.Add(image_wrapper);
			elements.Add(alignment);
			
			Gtk.Label header = new Gtk.Label();
			header.Markup = "<big>" + String.Format(Mono.Unix.Catalog.GetString("Search results for: {0}"),  "<b>" + search_text + "</b></big>");
			
			elements.PackStart(header, false, false, 0);
			elements.PackStart(new Gtk.HSeparator(), false, false, 0);
			
			Gtk.EventBox border_wrapper = new Gtk.EventBox();

			Gtk.Frame border = new Gtk.Frame();

			Gtk.Alignment border_alignment = new Gtk.Alignment(1, 1, 1, 1);
			border_alignment.BottomPadding = 5;
			border_alignment.TopPadding = 5;
			border_alignment.LeftPadding = 5;
			border_alignment.RightPadding = 5;
			
			border_alignment.Add(elements);

			border.Add(border_alignment);
			
			border_wrapper.Add(border);

			GtkCommon.set_background_color(border_wrapper, "white");

			popup.Add(border_wrapper);
		}

		private void add_link(string text, CalendarDriver.View view, System.DateTime item_modified_date, Gtk.HBox links)
		{
			Gtk.EventBox link_wrapper = new Gtk.EventBox();
			Gtk.Label link = new Gtk.Label();
			link.Markup = "<u><small>" + text + "</small></u>";
			
			GtkCommon.show_hand_and_tooltip(link_wrapper, text);
			
			link_wrapper.Add(link);
			link_wrapper.ButtonPressEvent += delegate(object sender, Gtk.ButtonPressEventArgs args) {
				set_view_callback(view, item_modified_date); 
			};
			GtkCommon.set_background_color(link_wrapper, "white");
			links.PackStart(link_wrapper, false, false, 5);
		}

		private Gtk.HBox create_element(Item item, string search_text, string snippet_text, VoidFunction update)
		{
			Gtk.HBox element = new Gtk.HBox();
			Gtk.VBox lines = new Gtk.VBox();
			Gtk.HBox title_date = new Gtk.HBox();

			Gtk.EventBox title_wrapper = new Gtk.EventBox();
			Gtk.Label title = new Gtk.Label();
			
			string str_title = item.long_name;
			
			if (str_title.Contains(search_text)) {
				int index = str_title.IndexOf(search_text);
				str_title = str_title.Substring(0, index) + "<b>" + search_text + "</b>" + str_title.Substring(index+search_text.Length);
			}
			
			title.Markup = "<big>" + str_title + "</big>";
			title.UseUnderline = false;
			title.SetAlignment(0, 0); // left aligned
			title_wrapper.ButtonPressEvent += delegate(object sender, Gtk.ButtonPressEventArgs args) {
				item.open(); 
			};
			title_wrapper.Add(title);
			GtkCommon.set_background_color(title_wrapper, "white");

			GtkCommon.show_hand_and_tooltip(title_wrapper, String.Format(Mono.Unix.Catalog.GetString("Open: {0}"), item.long_name));

			GtkCommon.create_preview(item, lines, title_wrapper);

			title_date.PackStart(title_wrapper, true, true, 2);
			
			Gtk.Label date = new Gtk.Label();
			date.Markup = "<big>" + String.Format("{0:dd/MM/yy}", item.last_modify) + "</big>";
			date.SetAlignment(1, 0); // right aligned
			title_date.PackStart(date, true, true, 0);

			lines.PackStart(title_date, true, true, 0);
			
			try {
				DocumentItem doc_item = item as DocumentItem;

				if (doc_item != null) {
					doc_item.on_snippet_callback = delegate (string snippet) {
						if (snippet.Length != 0 && snippet != "") {
							System.Console.WriteLine("got text: {0}", snippet);

							Gtk.Label snippet_label = new Gtk.Label();
							snippet_label.SetAlignment(0, 0); // left aligned
							snippet_label.Markup = snippet;
							lines.PackStart(snippet_label, true, true, 2);
							lines.ReorderChild(snippet_label, 1);

							update();
						}
					}; 
				}
			} catch (Exception) { // not document item
			}

			Gtk.HBox label_links = new Gtk.HBox();

			Gtk.HBox bubbles = new Gtk.HBox();

			foreach (UserLabel label in item.labels)
				bubbles.Add(label.dot());

			label_links.PackStart(bubbles, false, false, 0);

			item.on_labels_callback = delegate() {  
				foreach (UserLabel label in item.labels)
					bubbles.Add(label.dot());
					
				element.ShowAll();
			};

			Gtk.Alignment links_alignment = new Gtk.Alignment(1,1,0,0);
			Gtk.HBox links = new Gtk.HBox();

			add_link(Mono.Unix.Catalog.GetString("Go to week"), CalendarDriver.View.Week, item.last_modify, links);
			add_link(Mono.Unix.Catalog.GetString("Go to month"), CalendarDriver.View.Month, item.last_modify, links);
			add_link(Mono.Unix.Catalog.GetString("Go to year"), CalendarDriver.View.Year, item.last_modify, links);

			links_alignment.Add(links);

			label_links.PackStart(links_alignment, true, true, 0);
			
			lines.PackStart(label_links, true, true, 2);

			string image = "";
			
			if (item.file != null)
				image = Singleton<Types>.Instance.get_type((int)item.file.type.category).icon;

			Gtk.Alignment img_alignment = new Gtk.Alignment(0, (float)0.05, 0,0);

			if (image != "") 
			{
				System.Console.WriteLine("SEARCH POPUP image {0}", image);

				Gtk.Image img;
				
				if (item.file.starred)
					img = CairoDrawing.create_small_starred_image(image);
				else
					img = new Gtk.Image(null, image); 
				
				item.register_starred_change(delegate(bool starred) {
					foreach (Widget w in img_alignment)
						w.Destroy();
					if (starred)
						img = CairoDrawing.create_small_starred_image(image);
					else
						img = new Gtk.Image(null, image); 
					img_alignment.Add(img);
					img_alignment.ShowAll();
				});
				
				img_alignment.Add(img);
				img_alignment.ShowAll();
			}
			
			element.PackStart(img_alignment, false, false, 5);

			element.PackStart(lines, true, true, 0);

			return element;
		}
	}
}
