using System;
using Gtk;

namespace Nemo
{
	public class Tray
	{
		private MainWindow main_window;
	
		public bool hidden;

		public void restore_position()
		{
			main_window.Move(Singleton<Configuration>.Instance.data.main_window_x,
				             Singleton<Configuration>.Instance.data.main_window_y);
			Singleton<OverlayTracker>.Instance.show_all();
		}

		public void hide_window()
		{
			save_main_window_pos();
			main_window.hide();
			hidden = true;
		}

		public Tray(MainWindow window)
		{
			main_window = window;
			hidden = false;
			
			EventBox eb = new Gtk.EventBox();
			eb.Add(new Gtk.Image(null, "small_blue_guy.png"));
			eb.ButtonPressEvent += OnImageClick;
			Egg.TrayIcon icon = new Egg.TrayIcon("Test");
			icon.Add(eb);
			icon.ShowAll();	
		}
		
		private void OnImageClick(object o, ButtonPressEventArgs args)
		{
   			if (args.Event.Button == 3) //right click
   			{
      			Menu popupMenu = new Menu();

				// add show
      			ImageMenuItem menuPopup1;
      			if (hidden)
      				menuPopup1 = new ImageMenuItem(Mono.Unix.Catalog.GetString("Show window"));
				else
      				menuPopup1 = new ImageMenuItem(Mono.Unix.Catalog.GetString("Hide window"));
      			menuPopup1.Image = new Image(Stock.Refresh, IconSize.Menu); 
      			popupMenu.Add(menuPopup1); 
      			menuPopup1.Activated += delegate {
      				if (hidden)
      				{
						main_window.ShowAll();
						restore_position();
						hidden = false;
						main_window.Unref(); // restore correct reference count
      				}
      				else
      				{
						hide_window();
      				}
      			};

      			ImageMenuItem menuPopup2 = new ImageMenuItem(Mono.Unix.Catalog.GetString("About"));
      			menuPopup2.Image = new Image(Stock.About, IconSize.Menu); 
      			popupMenu.Add(menuPopup2); 
      			menuPopup2.Activated += delegate {
					Gtk.AboutDialog about = new AboutDialog();
					about.Name = "Nemo";
					about.Authors = new string[] { "Anders Rune Jensen", "Lau Bech Lauritzen", "Ole Laursen" };
					about.Artists = new string[] { "Linda Nhu", "Sune Theodorsen" };
					about.Comments = Mono.Unix.Catalog.GetString("Nemo is a file manager for those who would rather have their files manage themselves.");
					about.Logo = new Gdk.Pixbuf(null, "blue_guy_med.png");
					about.Response += delegate {
						about.Destroy();
					};
					about.Show();
      			};

				// add quit
      			ImageMenuItem menuPopup3 = new ImageMenuItem (Mono.Unix.Catalog.GetString("Quit"));
      			menuPopup3.Image = new Image(Stock.Quit, IconSize.Menu); 
      			popupMenu.Add(menuPopup3); 
      			menuPopup3.Activated += new EventHandler(this.OnPopupClick);
      			
				popupMenu.ShowAll(); 
      			popupMenu.Popup(null, null, null, args.Event.Button, args.Event.Time);
   			}
   			else if (args.Event.Button == 1 && hidden) 
   			{
				main_window.ShowAll();
				restore_position();
				hidden = false;
				main_window.Unref(); // restore correct reference count
   			}
			else if (args.Event.Button == 1 && !hidden) 
			{
				if (main_window.HasToplevelFocus)
					hide_window();
				else
					main_window.Present();
			}
		}

		private void save_main_window_pos()
		{
			int x, y, w, h; // used to restore position on the screen
			main_window.GetPosition(out x, out y);
			main_window.GetSize(out w, out h);
			Singleton<Configuration>.Instance.data.main_window_x = x;
			Singleton<Configuration>.Instance.data.main_window_y = y;
			Singleton<Configuration>.Instance.data.main_window_width = w;
			Singleton<Configuration>.Instance.data.main_window_height = h;
		}
		
		private void OnPopupClick(object o, EventArgs args)
		{
			if (!hidden)  
				save_main_window_pos();

			Singleton<Configuration>.Instance.save_configuration();
			Singleton<Broker>.Instance.stop();
			Application.Quit();
		}
	}
}
