using System;
using System.Collections.Generic;

using Mono.Unix;

namespace Nemo
{
	class DocumentItem : Item
	{
		private static string[] documents = { "application/msword", "application/vnd.oasis.opendocument.text", "application/pdf" }; 

		public static bool is_document(string mime)
		{
			foreach (string docs in documents)
				if (mime == docs)
					return true;
			return false;
		}

		private string pages;
		public string snippet;
		private string title;

		public delegate void OnSnippetResult(string snippet);
		public OnSnippetResult on_snippet_callback;
	
		public DocumentItem(File file, string[] fields, string search_text) : base(file)
		{
			on_snippet_callback = null;
		
			populate(search_text);

			parse_fields_and_update(fields);
		}
        
		public DocumentItem(string path, string[] fields, string search_text) : base(path, fields)
		{
			on_snippet_callback = null;
		
			populate(search_text);

			parse_fields_and_update(fields);
		}

        // Broker specific
        public static string[] ItemTypes()
        {
        	return new string[] { "Doc:Title", "Doc:PageCount" };
        }
        
		// used when parsing the fields array returned from broker
        static int fields_index;
        
        public static void set_fields_index(int index)
        {
        	fields_index = index;
        }
        
        private void parse_fields_and_update(string[] fields)
        {
			set_title(fields[fields_index]);

			display_item = new DisplayDocumentItem(pages, title, this);
			register_starred_change(display_item.update_starred);
			display_item.set_labels();
        }

        // set data
        
		protected void populate(string search_text)
		{
			if (search_text != "") {
				Singleton<Broker>.Instance.search_snippet(path, search_text, 
														  Helpers.RunInMainThread<string>(on_snippet_result));
			} else
				Singleton<Broker>.Instance.get_text(path, Helpers.RunInMainThread<string>(on_snippet_result));
 		}
        
        protected int set_page_count(string page_count)
        {
			if (page_count.Length == 0)
				return 0;
			else 
			{
				try 
				{
					string number = page_count.Replace(".0", ""); // hack
           		    pages = number;
            		return System.Convert.ToInt32(number);
            	} 
            	catch 
            	{
//            		System.Console.WriteLine("ERROR ERROR in page count from tracker: ->{0}<-", page_count);
           		    pages = "0";
            		return 0;
            	}
            }
		}

        protected void set_title(string t)
        {
        	if (t.Length != 0 && t != " " && t != "Microsoft Word - document.doc") {
				title = t;
				if (title.Length > 25) {
					title = title.Substring(0, 25);
					title += "...";
				}
			} else
				title = "";
		}
		
		protected void on_snippet_result(string text_snippet)
        {
        	snippet = text_snippet;
        
        	DisplayDocumentItem display_doc_item = (DisplayDocumentItem) display_item;
        
			string formatted_text = format_snippet_text(text_snippet, 150);
			display_doc_item.set_snippet(formatted_text);
			
			if (formatted_text.Length != text_snippet.Length)
				display_doc_item.set_snippet_tip(text_snippet);
				
			if (on_snippet_callback != null) {
				string small_formatted_text = format_snippet_text(text_snippet, 50);
				if (small_formatted_text != null && small_formatted_text != "")
					on_snippet_callback(small_formatted_text);
			}
        }

        // helper functions
        public string format_snippet_text(string text, int max_length)
        {
            if (text.Length < max_length)
            {
            	return text;
            }
            else
            {
                // find search word
                int word_begin = text.IndexOf("<b>");
                int word_end = text.IndexOf("</b>");

                if (word_begin == -1 || word_end == -1 || word_end <= word_begin || word_end - word_begin > max_length)
                {
//                    System.Console.WriteLine("word searched for not found in snippet {0}", text);
                    return "";
                }

                word_end += "</b>".Length;

                string small = text.Substring(word_begin, word_end - word_begin);

                int left_length = (max_length - small.Length) / 2;
                int right_length = (max_length - small.Length) / 2;

                if (left_length > word_begin)
                {
                    right_length += left_length - word_begin;
                    left_length = word_begin;
                }

                if (right_length > text.Length - word_end)
                {
                    if (left_length != word_begin)
                    {
                        left_length += right_length - (text.Length - word_end);
                        if (left_length > word_begin)
                            left_length = word_begin;
                    }
                    right_length = text.Length - word_end;
                }

                string left = text.Substring(word_begin - left_length, left_length);
                string right = text.Substring(word_end, right_length);

				return "..." + left + small + right + "...";
            }
        }
	}
}