using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

using Mono.Unix;

using System.Diagnostics; // for opening files

namespace Nemo
{
	// FIXME: pattern
	// pointer abstraction
	public class ItemWrapper
	{
		public Item item;
		
		public ItemWrapper(Item item)
		{
			this.item = item;
		}
	}

	public class Item
	{
		public DisplayItem Display()
		{
			return display_item;
		}

		public string mime_type;

		protected DisplayItem display_item;

		public File file; // FIXME: use this more
		
		public DateTime last_modify;

		public DateTime last_accessed;

		protected long long_size;
		
		public string long_name;

		public string name; // trimmed
		
		public string modify;
		public string accessed;
		public string size;
		
		public string path;

		public List<UserLabel> labels;

		public VoidFunction on_labels_callback;
		public VoidFunction on_got_data_from_tracker;

		public Item(File file)
		{
			this.file = file;

			setup(file.path);

			long_size = file.bytesize;
			set_size();
			
			last_modify = file.last_modified;
			last_accessed = file.last_accessed;
			
			modify = date_to_string(file.last_modified);
			accessed = date_to_string(file.last_accessed);
			
			name = long_name = file.path.Substring(file.path.LastIndexOf("/")+1); 
			
			string extension = "";
			int pos;
			if ((pos = name.LastIndexOf(".")) != -1)
				extension = name.Substring(pos);

			if (extension.Length > 4)
				extension = extension.Substring(0, 4);

			if (name.Length-extension.Length > 25-(6+extension.Length)) {
				name = name.Substring(0, 25-(6+extension.Length));
				name += "[...]" + extension;
			}
			
        	display_item = new DisplayItem(this);
			register_starred_change(display_item.update_starred);

			foreach (MetaLabel label in file.labels)
				labels.Add(new UserLabel(label));
			
			display_item.set_labels();
		}

		public void set_file(File file)
		{
			this.file = file;

			if (file.labels == null) {
				System.Console.WriteLine("path {0} not found in metadata store!", path);
				return;
			}

			foreach (MetaLabel label in file.labels)
				labels.Add(new UserLabel(label));

			display_item.set_labels();

			if (on_labels_callback != null)
				on_labels_callback();
		}

		public Item(string path, string[] fields)
		{
			setup(path);

			parse_fields_and_update(fields);
		}

		private void setup(string path)
		{
			// internal variables
			this.path = path;

			labels = new List<UserLabel>();
			starred_update_functions = new List<VoidFunction<bool>>();
		}

		public string image()
		{
			return Singleton<Types>.Instance.get_type((int)file.type.category).icon;
		}
		
		public void open()
		{
            Process.Start("'" + path + "'");
		}

        protected void parse_fields(string[] fields)
        {
        	System.Console.WriteLine("parsing fields");
        
        	foreach (string field in fields)
        		System.Console.WriteLine("'{0}'", field);

        	System.Console.WriteLine("=========");
        
        	long long_modify = System.Convert.ToInt64(fields[1]);
        	this.last_modify = Mono.Unix.Native.NativeConvert.FromTimeT(long_modify);

			long long_accessed = System.Convert.ToInt64(fields[2]);
        	last_accessed = Mono.Unix.Native.NativeConvert.FromTimeT(long_accessed);

			modify = date_to_string(last_modify);
			accessed = date_to_string(last_accessed);

			long_size = System.Convert.ToInt64(fields[3]);

			set_size();

			name = fields[4];

			long_name = name;

			string extension = "";
			int pos;
			if ((pos = name.LastIndexOf(".")) != -1)
				extension = name.Substring(pos);

			if (extension.Length > 4)
				extension = extension.Substring(0, 4);

			if (name.Length-extension.Length > 25-(6+extension.Length)) {
				name = name.Substring(0, 25-(6+extension.Length));
				name += "[...]" + extension;
			}

			mime_type = fields[5];
        }
        
        private void parse_fields_and_update(string[] fields)
        {
        	parse_fields(fields);
        	display_item = new DisplayItem(this);
			register_starred_change(display_item.update_starred);
        }

		List<VoidFunction<bool>> starred_update_functions;

		public void register_starred_change(VoidFunction<bool> update_function)
		{
			starred_update_functions.Add(update_function);
		}

		public string small_representation()
		{
			return name;
		}
		
		public void update_starred()
		{
			file.starred = !file.starred;
			
			Singleton<Broker>.Instance.set_starred(file, file.starred);

			foreach (VoidFunction<bool> func in starred_update_functions)
				func(file.starred);
		}
		
		// labels
	    public void make_label(UserLabel label)
    	{
			// update own labels       		
   			labels.Add(label);

			display_item.set_labels();

			if (on_labels_callback != null)
				on_labels_callback();

			// update tracker
			Singleton<Broker>.Instance.add_label_to_file(file, label.metalabel);
    	}
		
		public VoidFunction search_func;
	
		public void remove_label(UserLabel label)
		{
			List<UserLabel> new_labels = new List<UserLabel>();
		
			foreach (UserLabel tmp_label in labels)
				if (tmp_label.metalabel.db_id != label.metalabel.db_id)
					new_labels.Add(tmp_label);
					
			labels = new_labels;
			
			display_item.set_labels();

			if (on_labels_callback != null)
				on_labels_callback();

			Singleton<Broker>.Instance.remove_label_from_file(file, label.metalabel);
		}

		public bool contains_label(UserLabel label)
		{
			return labels.Exists(delegate (UserLabel tmp_label) {
				return tmp_label.metalabel.db_id == label.metalabel.db_id;
			});
		}
		
		private void set_size()
		{
			if (long_size > 1000*1000*1000)
				size = String.Format("{0:f2} gb", long_size/(1000*1000*1000.0));
			else if (long_size > 1000*1000)
				size = String.Format("{0:f2} mb", long_size/(1000*1000.0));
			else
				size = String.Format("{0:f2} kb", long_size/1000.0);
		}
		
        private string date_to_string(DateTime date)
        {
			return String.Format("{0:HH:mm:ss} {0:dd/MM/yy}", date); // {0:HH:mm:ss} 
        }
	}
}